#include <stdio.h>
#include <stdlib.h>

typedef union _broj
{
   long i;
   float f;
} broj;

void print_bits(int x)
{
	//x = 101010000
	//prvo stampamo najlevlji bit 1_01010000
	//posle stampamo 0 1_0_1010000
	// 10_1_010000
	
	//kako da izdvojimo i-ti bit da li 0 ili 1?
	//pravimo masku koja je svuda 0, a samo na i-tom mestu 1
	//0000001000000
	//a posle radimo x & maska
	//kao rezultat mozemo da dobijemo:
	//ili 0000001000000 - znaci na i-tom mestu je 1
	//ili 0000000000000 - znaci na i-tom mestu je 0
	
	//vracamo se na zadatak
	//prva maska 100000000000
	//sledeca    010000000000
	//....
	//zadnja     000000000001
	
	//unsigned int maska = 1 << 31;
	int bit_count = 8 * sizeof(unsigned int);
	unsigned int maska = 1 << (bit_count - 1);
	
	int i;
	for(i = 0; i < bit_count; i++)
	{
		unsigned r = maska & x;
		if(r == 0)
			putchar('0');
		else
			putchar('1');
		maska = maska >> 1;
	}
	putchar('\n');
}

int jedinica_na_mestu_k(int x, int k)
{
	//gleda se k-to mesto sa desna na levo, pocevsi od 0!!!
	unsigned maska = 1 << k;
	unsigned r = maska & x;
	return r != 0;
}

int postavi_jedinicu_na_mestu_k(int x, int k)
{
	//ne menjamo x, vec kao rezultat vracamo x, sa postavljenom jedinicom
	
	//ako hocemo na i to mesto da postavimo 1
	//pravimo masku 000001000000 i radimo operaciju |
	
	unsigned maska = 1 << k;
	int r = maska | x;
	return r;	
}

int suma_bitova(int x)
{
	int bit_count = 8 * sizeof(unsigned int);
	unsigned int maska = 1 << (bit_count - 1);
	int suma = 0;
	int i;
	for(i = 0; i < bit_count; i++)
	{
		unsigned r = maska & x;
		if(r != 0)
			suma++;
		maska = maska >> 1;
	}
	return suma;
}

unsigned get_bits(int x, int p, int n)
{
	//x = 00101011 | 10101101 | 01010111
	//p = 8, n = 8
	//y = 00000000000000000 | 10101101
	
	//x = 00101011 | 10101101 | 01010111
	//m = 00000000 | 11111111 | 00000000
	//x&m = 00000000 | 10101101 | 00000000
	//x&m >> p = 00000000000000000 | 10101101
	
	//y = x & m >> p
	
	//m = 0000000000000000
	//m = ~m -> 11111111111111
	//m = m << n -> 1111111100000
	//m = ~m -> 000000000111111
	//m = m << p -> 000001111100000
	
	unsigned maska = (~((~0)<<n))<<p;
	unsigned y = maska & x;
	return y >> p;
	
	//return (maska & x) >> p;
}

unsigned set_bits(unsigned x, int p, int n, unsigned y)
{
	//x = 10100101 | 110101 | 110100
	//y = 0000000000000000 | 111111
	//r = 10100101 | 111111 | 110100
	
	//proces je sledeci
	//moramo prvo da izdvojima zadnjih n bitova iz y
	//a odnda da ih pomerimo za p mesta ulevo, da se porvanjaju
	//sa odgovorajucim bitovima iz x
	//i onda y treba da izgleda ovako
	// 0000000| nbitova |0000000000
	
	//x = x1 | x2 | x3
	//u x treba da postavimo srednjih n bitova na 0
	//x = x1 | 0 | x3
	
	unsigned m;
	
	m = ~((~0) << n);
	//m = 0000000000011111
	
	y = y & m; //izdvojili smo zadnjih n bitova
	y = y << p; //poravnjali sa x
	
	m = ~((~((~0)<<n))<<p);
	//m = 11111100000111111
	
	x = x & m; //srednjih n bitova smo postavili na 0
	
	return x | y;
}

unsigned invert_bits(unsigned x, int p, int n)
{
	unsigned m = (~((~0)<<n))<<p;
	return m ^ x;
}
stamaj_znak_mantisu_eksponent(long x)
{
	//1 8 23
	int bit_count = 8 * sizeof(unsigned int);
	unsigned int maska = 1 << (bit_count - 1);
	
	int i;
	printf("znak: ");
	for(i = 0; i < 1; i++)
	{
		unsigned r = maska & x;
		if(r == 0)
			printf("0");
		else
			printf("1");
		maska = maska >> 1;
	}
	printf("\n");
	
	printf("eksponent: ");
	for(i = 0; i < 8; i++)
	{
		unsigned r = maska & x;
		if(r == 0)
			printf("0");
		else
			printf("1");
		maska = maska >> 1;
	}
	printf("\n");
	
	printf("mantisa: ");
	for(i = 0; i < 23; i++)
	{
		unsigned r = maska & x;
		if(r == 0)
			printf("0");
		else
			printf("1");
		maska = maska >> 1;
	}
	printf("\n");
}
int main()
{
	int x = 0x00FFAA00; //00000000111111111010101000000000
	int y = 0xFFFFFFFF;
	print_bits(x);
	
	printf("%d\n", jedinica_na_mestu_k(x, 0));
	printf("%d\n", jedinica_na_mestu_k(x, 18));
	
	print_bits(postavi_jedinicu_na_mestu_k(x, 2));
	
	printf("suma bitova: %d\n", suma_bitova(x));
	print_bits(get_bits(x, 6, 13));
	
	print_bits(set_bits(x, 2, 5, y));
	
	print_bits(invert_bits(x, 5, 24));
	
	printf("%d %d\n", sizeof(int), sizeof(float));
	broj b;
	scanf("%f", &b.f);
	stamaj_znak_mantisu_eksponent(b.i);
	
	return 0;
}